// --------------------------------------------------------------------------
#ifndef TDirectDrawClipperH
#define TDirectDrawClipperH

// ==========================================================================
// File:            TDirectDrawClipper.H
// Description:     This file contains the definition for the
//                  TDirectDrawClipper component
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// include the BCB related definitions, etc.
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>

// --------------------------------------------------------------------------
// include the basic TDirectDraw definitions, etc.
// --------------------------------------------------------------------------
#include "TDirectDrawInclude.h"
#include "TDirectDrawDefs.h"
#include "TDirectDrawErrors.h"

// ==========================================================================
//
// Class:           TDirectDrawClipper
//
// Description:     This is a component that wraps the functionality of the
//                  IDIRECTDRAWCLIPPER interface available within DirectX5
//
//                  All methods of this object attempt to call the relevant
//                  IDIRECTDRAWCLIPPER interface functions
//
//                  All methods call the OnError event method if any errors
//                  are encountered during execution of the relevant DirectX
//                  functions
//
// Published
// Properties:
//
// --------------------------------------------------------------------------
//
// Public           Created
// Properties:
//
// --------------------------------------------------------------------------
//
// Methods:         GetClipList
//                  SetClipList
//                  GetHWnd
//                  SetHWnd
//                  Initialize
//                  IsClipListChanged
// --------------------------------------------------------------------------
//
// Events:          OnCreate
//                  OnError
//                  OnMessage
//                  OnRelease
// ==========================================================================
class TDirectDrawClipper : public TComponent
{
__published:

// --------------------------------------------------------------------------
// Event:           OnCreate
// Description:     This event method is called when the Create() method has
//                  created the IDIRECTDRAWCLIPPER interface successfully
// --------------------------------------------------------------------------
    __property      TDirectDrawEvent       OnCreate = { read=FOnCreate,
                                                        write=FOnCreate,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnError
// Description:     This event method is called when a valid DirectX error
//                  occurs
// Note:            TDirectDrawClipper supplies a default OnError event
//                  handler which opens a simple message box to display the
//                  details of the error
//                  To stop TDirectDrawClipper Error messages from occurring,
//                  set this property to NULL after you call Create()
// --------------------------------------------------------------------------
    __property	    TDirectDrawError	    OnError = { read=FOnError,
                                                        write=FOnError,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnMessage
// Description:     This event method is called when the TDirectDraw object
//                  performs any kind of task. This is incredibly useful for
//                  debugging
// Note:            TDirectDrawClipper supplies a default OnMessage event
//                  handler which opens a simple message box to display the
//                  details of the error
//                  To stop TDirectDrawClipper messages from occurring, set
//                  this property to NULL after you call Create()
// --------------------------------------------------------------------------
    __property      TDirectDrawMessage    OnMessage = { read=FOnMessage,
                                                        write=FOnMessage,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnRelease
// Description:     This event method is called before the Release() method
//                  releases the IDIRECTDRAWCLIPPER interface
// --------------------------------------------------------------------------
    __property      TDirectDrawEvent      OnRelease = { read=FOnRelease,
                                                        write=FOnRelease,
                                                        nodefault };

public:

// --------------------------------------------------------------------------
// Property:        Created
// Description:     Indicates whether the create method was successful
// --------------------------------------------------------------------------
    __property      bool                    Created = { read=fCreated };

// --------------------------------------------------------------------------
// Constructor & Destructor
// --------------------------------------------------------------------------
	                __fastcall TDirectDrawClipper(TComponent* Owner);
    virtual			__fastcall	~TDirectDrawClipper();

// --------------------------------------------------------------------------
// Method:          GetClipList
// Description:     Retrieves a copy of the clip list associated with a
//                  DirectDrawClipper object
// Parameters:      ClipList    - Address of an RGNDATA structure that will
//                              contain the resulting copy of the clip list
//                              If this parameter is NULL, the method fills
//                              the variable at lpdwSize to the number of
//                              bytes necessary to hold the entire clip list
//                  Rect        - Address of a rectangle that will be used to
//                              clip the clip list. This parameter can be
//                              NULL to retrieve the entire clip list
//                  Size        - Size of the resulting clip list. When
//                              retrieving the clip list, this parameter is
//                              the size of the buffer at ClipList. When
//                              ClipList is NULL, the variable at lpdwSize
//                              receives the required size of the buffer, in
//                              bytes
// Note:            A subset of the clip list can be selected by passing a
//                  rectangle that clips the clip list
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	GetClipList();
    // LPRECT lpRect, LPRGNDATA lpClipList, LPDWORD lpdwSize

// --------------------------------------------------------------------------
// Method:          SetClipList
// Description:     Sets or deletes the clip list used by the Blt, BltBatch,
//                  and UpdateOverlay methods on surfaces to which the parent
//                  Clipper object is attached
// Parameters:      ClipList    - Either an address to a valid RGNDATA
//                  structure or NULL. If there is an existing clip list
//                  associated with the DirectDrawClipper object and this
//                  value is NULL, the clip list will be deleted
//                  Flags       -  This parameter is currently not used and
//                              must be set to 0
// Note:            The clip list cannot be set if a window handle is already
//                  associated with the Clipper object
//                  The BltFast method cannot clip
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	SetClipList();
    //LPRGNDATA lpClipList, DWORD dwFlags

// --------------------------------------------------------------------------
// Method:          GetHWnd
// Description:     Retrieves the window handle previously associated with
//                  this DirectDrawClipper object by the SetHWnd method
// Parameters:      hWnd    - Address of the window handle previously
//                          associated with this Clipper object by the
//                          SetHWnd method
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	GetHWnd();
    // HWND FAR *lphWnd

// --------------------------------------------------------------------------
// Method:          SetHWnd
// Description:     Sets the window handle that will obtain the clipping
//                  information
// Parameters:      Flags   - This parameter is currently not used and must
//                          be set to 0
//                  hWnd    - Window handle that obtains the clipping
//                          information
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	SetHWnd();
    // DWORD dwFlags, HWND hWnd

// --------------------------------------------------------------------------
// Method:          Initialize
// Description:     Initializes a Clipper object that was created by using
//                  the CoCreateInstance COM function
// Parameters:      DD      - Address of the DirectDraw structure that
//                          represents the DirectDraw object. If this
//                          parameter is set to NULL, an independent
//                          DirectDrawClipper object is created
//                  Flags   - This parameter is currently not used and must
//                          be set to 0
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	Initialize();
    // LPDIRECTDRAW lpDD, DWORD dwFlags

// --------------------------------------------------------------------------
// Method:          IsClipListChanged
// Description:     Monitors the status of the clip list if a window handle
//                  is associated with a DirectDrawClipper object
// Parameters:      Changed - Address of a variable that is set to TRUE if
//                          the clip list has changed
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	IsClipListChanged();
    // BOOL FAR *lpbChanged

protected:

private:

    TDirectDrawEvent            FOnCreate;
    TDirectDrawEvent            FOnRelease;
    TDirectDrawMessage          FOnMessage;
    TDirectDrawError	        FOnError;

	HRESULT			            fError;
    bool                        fCreated;

};
// --------------------------------------------------------------------------
#endif



