// --------------------------------------------------------------------------
#ifndef TDirectDrawPaletteH
#define TDirectDrawPaletteH

// ==========================================================================
// File:            TDirectDrawPalette.H
// Description:     This file contains the definition for the
//                  TDirectDrawPalette component
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// include the BCB related definitions, etc.
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>

// --------------------------------------------------------------------------
// include the basic TDirectDraw definitions, etc.
// --------------------------------------------------------------------------
#include "TDirectDrawInclude.h"
#include "TDirectDrawDefs.h"
#include "TDirectDrawErrors.h"

// ==========================================================================
//
// Class:           TDirectDrawPalette
//
// Description:     This is a component that wraps the functionality of the
//                  IDIRECTDRAWPALETTE interface available within DirectX5
//
//                  All methods of this object attempt to call the relevant
//                  IDIRECTDRAWPALETTE interface functions
//
//                  All methods call the OnError event method if any errors
//                  are encountered during execution of the relevant DirectX
//                  functions
//
// Published
// Properties:
//
// --------------------------------------------------------------------------
//
// Public           Created
// Properties:
//
// --------------------------------------------------------------------------
//
// Methods:         GetCaps
//                  GetEntries
//                  SetEntries
//                  Initialize
//
// --------------------------------------------------------------------------
//
// Events:          OnCreate
//                  OnError
//                  OnMessage
//                  OnRelease
// ==========================================================================
class TDirectDrawPalette : public TComponent
{
__published:

// --------------------------------------------------------------------------
// Event:           OnCreate
// Description:     This event method is called when the Create() method has
//                  created the IDIRECTDRAWPALETTE interface successfully
// --------------------------------------------------------------------------
    __property      TDirectDrawEvent       OnCreate = { read=FOnCreate,
                                                        write=FOnCreate,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnError
// Description:     This event method is called when a valid DirectX error
//                  occurs
// Note:            TDirectDrawPalette supplies a default OnError event
//                  handler which opens a simple message box to display the
//                  details of the error
//                  To stop TDirectDrawPalette Error messages from occurring,
//                  set this property to NULL after you call Create()
// --------------------------------------------------------------------------
    __property	    TDirectDrawError	    OnError = { read=FOnError,
                                                        write=FOnError,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnMessage
// Description:     This event method is called when the TDirectDraw object
//                  performs any kind of task. This is incredibly useful for
//                  debugging
// Note:            TDirectDrawPalette supplies a default OnMessage event
//                  handler which opens a simple message box to display the
//                  details of the error
//                  To stop TDirectDrawPalette messages from occurring, set
//                  this property to NULL after you call Create()
// --------------------------------------------------------------------------
    __property      TDirectDrawMessage    OnMessage = { read=FOnMessage,
                                                        write=FOnMessage,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnRelease
// Description:     This event method is called before the Release() method
//                  releases the IDIRECTDRAWPALETTE interface
// --------------------------------------------------------------------------
    __property      TDirectDrawEvent      OnRelease = { read=FOnRelease,
                                                        write=FOnRelease,
                                                        nodefault };

public:

// --------------------------------------------------------------------------
// Property:        Created
// Description:     Indicates whether the create method was successful
// --------------------------------------------------------------------------
    __property      bool                    Created = { read=fCreated };

// --------------------------------------------------------------------------
// Constructor & Destructor
// --------------------------------------------------------------------------
	                __fastcall TDirectDrawPalette(TComponent* Owner);
    virtual			__fastcall	~TDirectDrawPalette();

// --------------------------------------------------------------------------
// Method:          GetCaps
// Description:     Retrieves the capabilities of this palette object
// Parameters:      Caps    - Flag from the dwPalCaps member of the DDCAPS
//                          structure that defines palette capabilities
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	GetCaps();
    // LPDWORD lpdwCaps

// --------------------------------------------------------------------------
// Method:          GetEntries
// Description:     Queries palette values from a DirectDrawPalette object
// Parameters:      Base        - Start of the entries that should be
//                              retrieved sequentially
//                  Entries     - Address of the palette entries. Each field
//                              is a color description
//                  Flags       - This parameter is currently not used and
//                              must be set to 0
//                  NumEntries  - Number of palette entries that can fit in
//                              the address specified in Entries. The
//                              colors of each palette entry are returned in
//                              sequence, from the value of the StartingEntry
//                              parameter through the value of the Count
//                              parameter minus 1 (These parameters are set
//                              by the SetEntries method)
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	GetEntries();
    // DWORD dwFlags, DWORD dwBase, DWORD dwNumEntries,
    // LPPALETTEENTRY lpEntries

// --------------------------------------------------------------------------
// Method:          SetEntries
// Description:     Changes entries in a Palette object immediately
// Parameters:      Count           - Number of palette entries to be changed
//                  Entries         - Address of the palette entries. Each
//                                  field is a color description
//                  Flags           - This parameter is currently not used
//                                  and must be set to 0
//                  StartingEntry   - First entry to be set
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	SetEntries();
    // DWORD dwFlags, DWORD dwStartingEntry, DWORD dwCount,
    // LPPALETTEENTRY lpEntries


// --------------------------------------------------------------------------
// Method:          Initialize
// Description:     Initializes the TDirectDrawPalette object
// Parameters:      ColorTable  - This parameter is currently not used and
//                              must be set to 0
//                  DD          - Address of the DirectDraw structure that
//                              represents the DirectDraw object
//                  Flags       - This parameter is currently not used and
//                              must be set to 0
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	Initialize();
    // LPDIRECTDRAW lpDD, DWORD dwFlags, LPPALETTEENTRY lpDDColorTable

protected:

private:

    TDirectDrawEvent            FOnCreate;
    TDirectDrawEvent            FOnRelease;
    TDirectDrawMessage          FOnMessage;
    TDirectDrawError	        FOnError;

	HRESULT			            fError;
    bool                        fCreated;

};
//---------------------------------------------------------------------------
#endif
