// --------------------------------------------------------------------------
#ifndef TDirectDrawVideoPortH
#define TDirectDrawVideoPortH

// ==========================================================================
// File:            TDirectDrawVideoPort.H
// Description:     This file contains the definition for the
//                  TDirectDrawVideoPort component
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// include the BCB related definitions, etc.
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>

// --------------------------------------------------------------------------
// include the basic TDirectDraw definitions, etc.
// --------------------------------------------------------------------------
#include "TDirectDrawInclude.h"
#include "TDirectDrawDefs.h"
#include "TDirectDrawErrors.h"

// ==========================================================================
//
// Class:           TDirectDrawVideoPort
//
// Description:     This is a component that wraps the functionality of the
//                  IDIRECTDRAWVIDEOPORT interface available within DirectX5
//
//                  All methods of this object attempt to call the relevant
//                  IDIRECTDRAWVIDEOPORT interface functions
//
//                  All methods call the OnError event method if any errors
//                  are encountered during execution of the relevant DirectX
//                  functions
//
// Published
// Properties:
//
// --------------------------------------------------------------------------
//
// Public           Created
// Properties:
//
// --------------------------------------------------------------------------
//
// Methods:         Flip
//                  GetBandwidthInfo
//                  GetColorControls
//                  GetFieldPolarity
//                  GetInputFormats
//                  GetOutputFormats
//                  GetVideoLine
//                  GetVideoSignalStatus
//                  SetColorControls
//                  SetTargetSurface
//                  StartVideo
//                  StopVideo
//                  UpdateVideo
//                  WaitForSync
//
// --------------------------------------------------------------------------
//
// Events:          OnCreate
//                  OnError
//                  OnMessage
//                  OnRelease
// ==========================================================================
class TDirectDrawVideoPort : public TComponent
{
__published:

// --------------------------------------------------------------------------
// Event:           OnCreate
// Description:     This event method is called when the Create() method has
//                  created the IDIRECTDRAWVIDEOPORT interface successfully
// --------------------------------------------------------------------------
    __property      TDirectDrawEvent       OnCreate = { read=FOnCreate,
                                                        write=FOnCreate,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnError
// Description:     This event method is called when a valid DirectX error
//                  occurs
// Note:            TDirectDrawVideoPort supplies a default OnError event
//                  handler which opens a simple message box to display the
//                  details of the error
//                  To stop TDirectDrawVideoPort Error messages from
//                  occurring, set this property to NULL after you call
//                  Create()
// --------------------------------------------------------------------------
    __property	    TDirectDrawError	    OnError = { read=FOnError,
                                                        write=FOnError,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnMessage
// Description:     This event method is called when the TDirectDraw object
//                  performs any kind of task. This is incredibly useful for
//                  debugging
// Note:            TDirectDrawVideoPort supplies a default OnMessage event
//                  handler which opens a simple message box to display the
//                  details of the error
//                  To stop TDirectDrawVideoPort messages from occurring, set
//                  this property to NULL after you call Create()
// --------------------------------------------------------------------------
    __property      TDirectDrawMessage    OnMessage = { read=FOnMessage,
                                                        write=FOnMessage,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnRelease
// Description:     This event method is called before the Release() method
//                  releases the IDIRECTDRAWVIDEOPORT interface
// --------------------------------------------------------------------------
    __property      TDirectDrawEvent      OnRelease = { read=FOnRelease,
                                                        write=FOnRelease,
                                                        nodefault };

public:

// --------------------------------------------------------------------------
// Property:        Created
// Description:     Indicates whether the create method was successful
// --------------------------------------------------------------------------
    __property      bool                    Created = { read=fCreated };

// --------------------------------------------------------------------------
// Constructor & Destructor
// --------------------------------------------------------------------------
	                __fastcall TDirectDrawVideoPort(TComponent* Owner);
    virtual			__fastcall	~TDirectDrawVideoPort();

// --------------------------------------------------------------------------
// Method:          Flip
// Description:     Instructs the VideoPort object to write the next frame of
//                  video to a new surface
// Parameters:      Flags   - Flip options flags
//                  Surface - Address of the DirectDrawSurface object that
//                          will receive the next frame of video
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall      Flip();
    // LPDIRECTDRAWSURFACE lpDDSurface, DWORD dwFlags

// --------------------------------------------------------------------------
// Method:          GetBandwidthInfo
// Description:     Retrieves the minimum required overlay zoom factors and
//                  device limitations of a video port that uses the provided
//                  output pixel format
// Parameters:      Bandwidth   - Address of a DDVIDEOPORTBANDWIDTH structure
//                              that will be filled with the retrieved
//                              bandwidth and device dependency information
//                  Flags       - Indicates how the method is to interpret
//                              the Width and Height parameters
//                  Format      - Address of a DDPIXELFORMAT structure that
//                              describes the output pixel format for which
//                              bandwidth information will be retrieved
//                  Height      - Vertical dimension of an overlay or video
//                              data. The interpretation of this parameter
//                              depends on the value specified in the dwFlags
//                              parameter
//                  Width       - Horizontal dimension of an overlay or video
//                              data. The interpretation of this parameter
//                              depends on the value specified in the Flags
//                              parameter
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall      GetBandwidthInfo();
    // LPDDPIXELFORMAT lpddpfFormat, DWORD dwWidth, DWORD dwHeight,
    // DWORD dwFlags, LPDDVIDEOPORTBANDWIDTH lpBandwidth


// --------------------------------------------------------------------------
// Method:          GetColorControls
// Description:     Returns the current color control settings associated
//                  with the video port
// Parameters:      ColorControl    - Address of a DDCOLORCONTROL structure
//                                  that will be filled with the current
//                                  settings of the video port's color
//                                  control
// Note:            The Flags member of the DDCOLORCONTROL structure indicate
//                  which of the color control options are supported
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall      GetColorControls();
    // LPDDCOLORCONTROL lpColorControl

// --------------------------------------------------------------------------
// Method:          GetFieldPolarity
// Description:     Retrieves the polarity of a video field
// Parameters:      FieldPolarity   - Address of a variable that will be set
//                                  to indicate the current field polarity.
//                                  This value is set to true if the current
//                                  video field is the even field of an
//                                  interlaced video signal and false
//                                  otherwise
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall      GetFieldPolarity();
    // LPBOOL lpbFieldPolarity

// --------------------------------------------------------------------------
// Method:          GetInputFormats
// Description:     Retrieves the input formats supported by the VideoPort
//                  object
// Parameters:      Flags       - Specifies the part of the video signal for
//                              which formats will be enumerated
//                  Formats     - Address of an array of DDPIXELFORMAT
//                              structures that will be filled in with the
//                              input formats supported by this VideoPort
//                              object. If this parameter is NULL, the method
//                              sets NumFormats to the number of supported
//                              formats and the returns DD_OK
//                  NumFormats  - Address of a variable containing the number
//                              of entries that the array at Formats can hold
//                              If this number is less than the total number
//                              of codes, the method fills the array with as
//                              many codes as will fit, sets the value at
//                              NumFormats to indicate the total number of
//                              codes, and returns DDERR_MOREDATA
// Note:            This method can also be used to return the number of
//                  formats supported. To do this, set the Formats parameter
//                  to NULL. When the method returns, the variable at
//                  NumFormats contains the total number of supported input
//                  formats
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall      GetInputFormats();
    // LPDWORD lpNumFormats, LPDDPIXELFORMAT lpFormats, DWORD dwFlags

// --------------------------------------------------------------------------
// Method:          GetOutPutFormats
// Description:     Retrieves a list of output formats that the VideoPort
//                  object supports for a specified input format
// Parameters:      Flags       - Specifies the part of the video signal for
//                              which formats will be enumerated
//                  Formats     - Address of an array of DDPIXELFORMAT
//                              structures that will be filled in with the
//                              output formats supported by this VideoPort
//                              object. If this parameter is NULL, the method
//                              sets NumFormats to the number of supported
///                             formats and the returns DD_OK
//                  InputFormat - Address of a DDPIXELFORMAT structure that
//                              describes the input format for which
//                              conversion information is requested
//                  NumFormats  - Address of a variable containing the number
//                              of entries that the array at Formats can hold
//                              If this number is less than the total number
//                              of codes, the method fills the array with as
//                              many codes as will fit, sets the value at
//                              NumFormats to indicate the total number of
//                              codes, and returns DDERR_MOREDATA
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall      GetOutputFormats();
    // LPDDPIXELFORMAT lpInputFormat, LPDWORD lpNumFormats,
    // LPDDPIXELFORMAT lpFormats, DWORD dwFlags

// --------------------------------------------------------------------------
// Method:          GetVideoLine
// Description:     Retrieves the current line of video being written to the
//                  frame buffer
// Parameters:      Line    - Address of a variable that will be filled with
//                          a value indicating the video line currently being
//                          written to the frame buffer
// Note:            The value this method retrieves reflects the true video
//                  line being written, relative to the field height, before
//                  any prescaling occurs
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall      GetVideoLine();
    // LPDWORD lpdwLine

// --------------------------------------------------------------------------
// Method:          GetVideoSignalStatus
// Description:     Retrieves the status of the video signal currently being
//                  presented to the video port
// Parameters:      Status  - Address of a variable that will contain a
//                          return code indicating the quality of the video
//                          signal at the video port
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall      GetVideoSignalStatus();
    // LPDWORD lpdwStatus

// --------------------------------------------------------------------------
// Method:          SetColorControls
// Description:     Sets the color control settings associated with the video
//                  port
// Parameters:      ColorControl    - Address of a DDCOLORCONTROL structure
//                                  containing the new color control settings
//                                  that will be applied to the video port
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall      SetColorControls();
    //  LPDDCOLORCONTROL lpColorControl

// --------------------------------------------------------------------------
// Method:          SetTargetSurface
// Description:     Sets the DirectDraw surface object that will receive the
//                  stream of live video data and/or the vertical blank
//                  interval data
// Parameters:      Flags   - Value specifying the type of target surface
//                  Surface - Address of the Surface object that will receive
//                          the video data
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall      SetTargetSurface();
    // LPDIRECTDRAWSURFACE lpDDSurface, DWORD dwFlags

// --------------------------------------------------------------------------
// Method:          StartVideo
// Description:     Enables the hardware video port and starts the flow of
//                  video data into the currently specified surface
// Parameters:      VideoInfo   - Address of a pointer to a DDVIDEOPORTINFO
//                              structure
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall      StartVideo();
    // LPDDVIDEOPORTINFO lpVideoInfo

// --------------------------------------------------------------------------
// Method:          StopVideo
// Description:     Stops the flow of video port data into the frame buffer
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall      StopVideo();

// --------------------------------------------------------------------------
// Method:          UpdateVideo
// Description:     Updates parameters that govern the flow of video data
//                  from the video-port to the Surface object
// Parameters:      VideoInfo   -   Address of a DDVIDEOPORTINFO structure
//                              that describes the video transfer parameters
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall      UpdateVideo();
    // LPDDVIDEOPORTINFO lpVideoInfo

// --------------------------------------------------------------------------
// Method:          WaitForSync
// Description:     Waits for VSYNC or until a given scan line is being drawn
// Parameters:      Flags   - Specifies how the method will wait for the
//                          video VSYNC or the specified line number
//                  Line    - The video line determining when the method
//                          should return, relative to the field height,
//                          before prescaling. This parameter is ignored if
//                          the Flags parameter is set to
//                          DDVPWAIT_BEGINVBLANK or DDVPWAIT_ENDVBLANK
//                  Timeout - Amount of time, in milliseconds, that the
//                          method will wait for the next video vertical
//                          blank before timing out. If this parameter is 0,
//                          the method waits 3 times the value specified in
//                          the MicrosecondsPerField member of the
//                          DDVIDEOPORTDESC
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall      WaitForSync();
    //  DWORD dwFlags, DWORD dwLine, DWORD dwTimeout

protected:

private:

    TDirectDrawEvent            FOnCreate;
    TDirectDrawEvent            FOnRelease;
    TDirectDrawMessage          FOnMessage;
    TDirectDrawError	        FOnError;

	HRESULT			            fError;
    bool                        fCreated;

};
//---------------------------------------------------------------------------
#endif
