// --------------------------------------------------------------------------
#ifndef TDirectDrawVideoPortContainerH
#define TDirectDrawVideoPortContainerH

// ==========================================================================
// File:            TDirectDrawVideoPortContainer.H
// Description:     This file contains the definition for the
//                  TDirectDrawVideoPortContainer component
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// include the BCB related definitions, etc.
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>

// --------------------------------------------------------------------------
// include the basic TDirectDraw definitions, etc.
// --------------------------------------------------------------------------
#include "TDirectDrawInclude.h"
#include "TDirectDrawDefs.h"
#include "TDirectDrawErrors.h"

// ==========================================================================
//
// Class:           TDirectDrawVideoPortContainer
//
// Description:     This is a component that wraps the functionality of the
//                  IDIRECTDRAWVIDEOPORTCONTAINER interface available within
//                  DirectX5
//
//                  All methods of this object attempt to call the relevant
//                  IDIRECTDRAWVIDEOPORTCONTAINER interface functions
//
//                  All methods call the OnError event method if any errors
//                  are encountered during execution of the relevant DirectX
//                  functions
//
// Published
// Properties:
//
// --------------------------------------------------------------------------
//
// Public           Created
// Properties:
//
// --------------------------------------------------------------------------
//
// Methods:         CreateVideoPort
//                  EnumVideoPorts
//                  GetVideoPortConnectInfo
//                  QueryVideoPortStatus
//
// --------------------------------------------------------------------------
//
// Events:          OnCreate
//                  OnError
//                  OnMessage
//                  OnRelease
// ==========================================================================
class TDirectDrawVideoPortContainer : public TComponent
{
__published:
// --------------------------------------------------------------------------
// Event:           OnCreate
// Description:     This event method is called when the Create() method has
//                  created the IDIRECTDRAWVIDEOPORT interface successfully
// --------------------------------------------------------------------------
    __property      TDirectDrawEvent       OnCreate = { read=FOnCreate,
                                                        write=FOnCreate,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnError
// Description:     This event method is called when a valid DirectX error
//                  occurs
// Note:            TDirectDrawVideoPort supplies a default OnError event
//                  handler which opens a simple message box to display the
//                  details of the error
//                  To stop TDirectDrawVideoPort Error messages from
//                  occurring, set this property to NULL after you call
//                  Create()
// --------------------------------------------------------------------------
    __property	    TDirectDrawError	    OnError = { read=FOnError,
                                                        write=FOnError,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnMessage
// Description:     This event method is called when the TDirectDraw object
//                  performs any kind of task. This is incredibly useful for
//                  debugging
// Note:            TDirectDrawVideoPort supplies a default OnMessage event
//                  handler which opens a simple message box to display the
//                  details of the error
//                  To stop TDirectDrawVideoPort messages from occurring, set
//                  this property to NULL after you call Create()
// --------------------------------------------------------------------------
    __property      TDirectDrawMessage    OnMessage = { read=FOnMessage,
                                                        write=FOnMessage,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnRelease
// Description:     This event method is called before the Release() method
//                  releases the IDIRECTDRAWVIDEOPORT interface
// --------------------------------------------------------------------------
    __property      TDirectDrawEvent      OnRelease = { read=FOnRelease,
                                                        write=FOnRelease,
                                                        nodefault };

public:

// --------------------------------------------------------------------------
// Property:        Created
// Description:     Indicates whether the create method was successful
// --------------------------------------------------------------------------
    __property      bool                    Created = { read=fCreated };

// --------------------------------------------------------------------------
// Constructor & Destructor
// --------------------------------------------------------------------------
	                __fastcall  TDirectDrawVideoPortContainer
                                                        (TComponent* Owner);
    virtual			__fastcall	~TDirectDrawVideoPortContainer();

// --------------------------------------------------------------------------
// Method:          CreateVideoPort
// Description:     Creates a VideoPort object
// Parameters:      DDVideoPort     - Address of a variable that will be
//                                  filled with a pointer to a new VideoPort
//                                  object's IDirectDrawVideoPort interface
//                                  if the call succeeds
//                  DDVideoPortDesc - Address of a DDVIDEOPORTDESC structure
//                                  that describes the VideoPort object to be
//                                  created
//                  Flags           - Flags specifying video-port control
//                                  options
//                  UnkOuter        - Allows for future compatibility with
//                                  COM aggregation features. Presently,
//                                  however, this method will return an error
//                                  if this parameter is anything but NULL
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall  CreateVideoPort();
    // DWORD dwFlags, LPDDVIDEOPORTDESC lpDDVideoPortDesc,
    // LPDIRECTDRAWVIDEOPORT FAR *lplpDDVideoPort, IUnknown FAR *pUnkOuter

// --------------------------------------------------------------------------
// Method:          EnumVideoPorts
// Description:     Enumerates all of the video ports that the hardware
//                  exposes that are compatible with a provided video port
//                  description
// Parameters:      Context             - Address of a caller-defined
//                                      structure that will be passed to each
//                                      enumeration member
//                  DDVideoPortCaps     - Pointer to a DDVIDEOPORTCAPS
//                                      structure that will be checked
//                                      against the available video ports. If
//                                      this parameter is NULL, all video
//                                      ports will be enumerated
//                  EnumVideoCallback   - Address of the EnumVideoCallback
//                                      function that will be called each
//                                      time a match is found
//                  Flags               - Reserved for future use. This
//                                      parameter must be zero
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall  EnumVideoPorts();
    // DWORD dwFlags, LPDDVIDEOPORTCAPS lpDDVideoPortCaps, LPVOID lpContext,
    // LPENUMVIDEOCALLBACK lpEnumVideoCallback

// --------------------------------------------------------------------------
// Method:          GetVideoPortConnectInfo
// Description:     Retrieves the connection information supported by all
//                  video ports
// Parameters:      ConnectInfo - Address of an array of DDVIDEOPORTCONNECT
//                              structures that will be filled with the
//                              connection options supported by the specified
//                              video port. If this parameter is NULL, the
//                              method sets lpNumEntries to indicate the
//                              total number of connections that the video
//                              port supports, then returns DD_OK
//                  NumEntries  - Address of a variable containing the number
//                              of entries that the array at ConnectInfo can
//                              hold. If this number is less than the total
//                              number of connections, the method fills the
//                              array with as many entries as will fit, sets
//                              the value at NumEntries to indicate the total
//                              number of connections, and returns
//                              DDERR_MOREDATA
//                  PortId      - Video port ID of the video port for which
//                              the connection information will be retrieved
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall  GetVideoPortConnectInfo();
    // DWORD dwPortId, LPDWORD lpNumEntries,
    // LPDDVIDEOPORTCONNECT lpConnectInfo

// --------------------------------------------------------------------------
// Method:          QueryVideoPortStatus
// Description:
// Parameters:      PortId      - Video port ID of the video port for which
//                              the status information will be retrieved
//                  VPStatus    - Address of a DDVIDEOPORTSTATUS structure
//                              that will be filled with information about
//                              the status of the specified video port
// Note:            Supported in Windows 98 and Windows NT 5.0 only
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall  booga();
    // DWORD dwPortId, LPDDVIDEOPORTSTATUS lpVPStatus

protected:
private:

    TDirectDrawEvent            FOnCreate;
    TDirectDrawEvent            FOnRelease;
    TDirectDrawMessage          FOnMessage;
    TDirectDrawError	        FOnError;

	HRESULT			            fError;
    bool                        fCreated;

};
//---------------------------------------------------------------------------
#endif
